#!/usr/bin/env python

from decimal import Rounded
import os
import sys
from sys import argv

import libscores
import yaml

import pandas as pd
from sklearn.metrics import f1_score
import sklearn

import csv

from libscores import ls, filesep, mkdir, read_array, compute_all_scores, write_scores

# Default I/O directories:
root_dir = "/home/ritesh/Dropbox/kmi-event-related-stuffs/TRAC-2022/shared_task/TRAC-2022-Shared-Task-Final"
default_input_dir = os.path.join(root_dir, "reference_data_2")
default_output_dir = os.path.join(root_dir, "scores_2")

# Constant used for a missing score
missing_score = -0.999999

# Debug flag 0: no debug, 1: show all scores, 2: also show version amd listing of dir
debug_mode = 0


def validate_filename(pred_file, html_file):
    valid_filenames = ['trac_2022_pred.tsv', 'trac_sur_pred.tsv', 'trac_all_pred.tsv']
    if pred_file in valid_filenames:
        return True
    else:
        result_type = '''<b>Invalid Submission File Name:</b> %s <br><b>Expected File Names:</b> %s <br/><font color='blue'>The submission file name(s) should exactly match from this list</font>''' % (pred_file, str(valid_filenames))    
        
        html_txt = '''
                    <font color='red'>
                        <h3> Error in processing Submission </h3>
                    </font> <br/> %s
                    ''' % result_type
                
        html_file.write(html_txt)
        return False

def validate_input(input_columns, expected_columns, file_name, outfile):
    valid = True
    actual_columns = list(input_columns)

    error_txt = ''

    if 'trac_2022_' in file_name or 'trac_sep_' in file_name:
        if len(actual_columns) < 1 or len(actual_columns) > 7:
            error_txt = '''Number of columns for Task 1 should be between 2 - 8. Please check your file format. Ensure that it is in TSV format with different columns separated by TAB. The file should only contain IDs and the predictions - any additional columns should be removed.'''
            valid = False

    elif 'trac_all_' in file_name:
        if len(actual_columns) != 1:
            error_txt = '''Number of columns for Task 2 should be 2. Please check your file format. Ensure that it is in TSV format with different columns separated by TAB. The file should only contain IDs and the predictions - any additional columns should be removed.'''
            valid = False

    if valid:
        for actual_column in actual_columns:
            if actual_column not in expected_columns:
                col_error = '''The Column Names / header of the submission file must be one of the elements from this list and an ID column'''
                error_txt = error_txt + '''<b>Column Found:</b> %s <br/><b>Expected Columns:</b> %s<br/> <font color='blue'> %s </font><br/><br/>''' % (actual_column, str(expected_columns), col_error)
                valid = False


    if not valid:
        html_txt = '''
                    <font color='red'>
                        <h3> Error in processing %s </h3>
                    </font><br/> %s
                    ''' % (file_name, error_txt)
        outfile.write(html_txt)

    return valid


def process_f1 (gold, prediction):    
    gold_cols = gold.columns
    pred_col = prediction.columns[-1]

    # print ('9/16 Gold cols', gold_cols)
    # print ('9/16 pred cols', pred_col)
    # print ('9/16 All pred cols', prediction.columns)

    all_f1 = []
    f1_weights = []

    for gold_col in gold_cols:
        if pred_col in gold_col:
            cur_gold = gold[['ID', gold_col]]
            cur_gold = cur_gold[cur_gold[gold_col] != 'NONE']
            # df_merged = pd.merge(cur_gold, prediction, left_on='ID', right_on='ID')
            # print ('9/16 Length of current gold df', len(cur_gold), 'prediction head', len(prediction))
            # print (prediction.head())
            # print ('Gold head', len(cur_gold))
            # print (cur_gold.head())
            df_merged = pd.merge(cur_gold, prediction, left_on='ID', right_on='ID')
            # df_merged = pd.merge(cur_gold, prediction, left_on='ID')
            # print ('Merged head')
            # print (df_merged.head())
            df_merged.columns = ['ID', 'labels_true', 'labels_pred']
            df_merged.fillna('UDF', inplace=True)
            # print ('9/16 Length of merged df', len(df_merged))

            label_true = list(df_merged['labels_true'])
            label_pred = list(df_merged['labels_pred'])
            micro_f1_label = f1_score(label_true, label_pred, average='micro')
            all_f1.append(micro_f1_label)
            f1_weights.append(len(label_true))
    
    sum_f1 = 0.0
    all_f1_weights = sum(f1_weights)
    if all_f1_weights > 0:
        for cur_f1, cur_weight in zip(all_f1, f1_weights):
            wtd_f1 = cur_f1*cur_weight
            sum_f1 = sum_f1 + wtd_f1
        micro_f1_score = sum_f1 / (all_f1_weights)
        f1_weight = len(label_true)
        print ('9/16 Weighted f1', micro_f1_score)
    else:
        micro_f1_score = 0.0
        f1_weight = 0

    print ('Overall weighted f1', micro_f1_score)

    return micro_f1_score, f1_weight


def get_prefix(pred_name, pred_col):
    if 'trac_2022' in pred_name:        
        prefix = '2022_'+ score_prefix_col[pred_col]
        overall_prefix = '2022'
    else:
        if 'sur' in pred_name:
            prefix = 'surp_'+ score_prefix_col[pred_col]
            overall_prefix = 'surp'
    
    return prefix, overall_prefix


if __name__ == "__main__":

    print('1 The Python version is {}.'.format(sys.version))
    print('2 The scikit-learn version is {}.'.format(sklearn.__version__))

    score_prefix_fname = {
        'trac_2022_pred': '2022',
        'trac_sur_pred': 'sur',
        'trac_all_pred': ''
    }

    score_prefix_col = {
        'Aggression':'agg',
        'Aggression_Intensity':'agi',
        'Discursive_Role':'dsr',
        'Gender_Bias':'geb',
        'Communal_Bias':'cob',
        'Caste_Class_Bias':'cab',
        'Ethnicity_Racial_Bias':'reb'
    }

    col_names = list(score_prefix_col.keys())
    all_events = ['2018', '2020', '2022', 'covid']
    surprise_events = ['Regular', 'Surprise']

    #### INPUT/OUTPUT: Get input and output directory names
    if len(argv) == 1:  # Use the default input and output directories if no arguments are provided
        print ('3 Defaulting to default dirs')
        input_dir = default_input_dir
        output_dir = default_output_dir
    else:
        input_dir = argv[1]
        output_dir = argv[2]
        # Create the output directory, if it does not already exist and open output files
    print ('4 Input directory', input_dir)
    print ('5 Output directory', output_dir)

    mkdir(output_dir)
    score_file = open(os.path.join(output_dir, 'scores.txt'), 'w')
    html_file = open(os.path.join(output_dir, 'scores.html'), 'w')

    # Get all the prediction files from the solution directory
    solution_path = os.path.join(input_dir, 'ref')
    prediction_names = sorted(ls(os.path.join(input_dir, 'res', '*.tsv')))
  
    prefix = ''
    for pred_name in prediction_names:
        pred_file = pred_name.split('/')[-1]
        try:            
            valid_fname = validate_filename(pred_file, html_file)

            if valid_fname:
                print ('6 Predicted File Path', pred_name)
                if 'trac_sur_' in pred_name:
                    result_type = 'Task 1 Results with Surprise Language Dataset'
                elif 'trac_2022_' in pred_name:
                    result_type = 'Task 1 Results without Surprise Language Dataset'
                elif 'trac_all_' in pred_name:
                    result_type = 'Task 2 Results'

                html_txt = '''
                    <font color='red'>
                        <h3> %s </h3>
                    </font>
                    ''' % result_type
                
                html_file.write(html_txt)
                
                solution_prefix = pred_file.replace('pred', 'gold').replace('res', 'ref').replace('.tsv', '_')           

                pred_df = pd.read_csv(pred_name, sep='\t')
                pred_cols = pred_df.columns[1:]

                valid = validate_input(pred_cols, col_names, pred_file, html_file)

                if valid:
                    all_f1 = []
                    all_weights = []
                    if len(pred_cols) > 1:
                        for pred_col in pred_cols:
                            pred_col_name = pred_col.replace(' ', '_').replace('/', '_')
                            cur_solution_path = os.path.join(input_dir, 'ref', solution_prefix)+pred_col_name+'.tsv'
                            print ('8 Gold file path', cur_solution_path)
                            prediction = pred_df[['ID', pred_col]]
        
                            # print ('8b Getting prefix')
                            prefix, overall_prefix = get_prefix(pred_name, pred_col)
                            
                            gold = pd.read_csv(cur_solution_path, sep='\t')

                            # print ('9 Getting F1')
                            micro_f1_score, f1_weight = process_f1 (gold, prediction)
                            
                            print ('10 F1 Processed, writing to files')
                            rounded_f1 = round(micro_f1_score, 2)
                            html_file.write(
                            "<b> %s Micro F1: </b><br/> %0.2f <br/><br/>" % (pred_col, rounded_f1))

                            score_file.write(
                            "%s_microf1: %0.12f\n" % (prefix, micro_f1_score))

                            if 'trac_sur_' in pred_name:
                                for cur_event in surprise_events:
                                    
                                    gold_current = gold[gold['Event'] == cur_event]
                                    # print ('16a Total', cur_event, len(gold_current), len(gold))
                                    # print (gold_current.head())
                                    
                                    # print ('16 Getting Surprise F1')
                                    micro_f1_score, f1_weight = process_f1 (gold_current, prediction)

                                    rounded_f1 = round(micro_f1_score, 2)
                                    if cur_event == 'Regular':
                                        hcolor = 'turquoise'
                                    else:
                                        hcolor = 'orange'
                                    # print ('17 Surprise F1 Processed, writing to files')
                                    html_file.write(
                            "<b> %s Micro F1 on <font color='%s'>%s</font> Data: </b><br/> %0.2f <br/><br/>" % (pred_col, hcolor, cur_event, rounded_f1))

                            all_f1.append(micro_f1_score)
                            all_weights.append(f1_weight)
                    else:
                        all_f1 = []
                        all_weights = []
                        overall_prefix = 'task2'
                        solution_file = 'trac_all_gold_Aggression.tsv'
                        cur_solution_path = os.path.join(solution_path, solution_file)
                        print ('11 Gold file path', cur_solution_path)
                        
                        gold = pd.read_csv(cur_solution_path, sep='\t', dtype=str)

                        for cur_event in all_events:
                            prefix = cur_event+'_agg'
                            gold_current = gold[gold['Event'] == cur_event]
                            
                            # print (gold_current.head())
                            
                            print ('16 Getting F1')
                            micro_f1_score, f1_weight = process_f1 (gold_current, pred_df)
                            
                            print ('17 F1 Processed, writing to files')
                            rounded_f1 = round(micro_f1_score, 2)
                            html_file.write(
                            "<b> %s Aggression Micro F1:</b><br/> %0.2f <br/><br/>" % (cur_event, rounded_f1))


                            score_file.write(
                            "%s_agg_microf1: %0.2f\n" % (overall_prefix+'_'+cur_event, micro_f1_score))

                            all_f1.append(micro_f1_score)
                            all_weights.append(f1_weight)

                    
                    sum_f1 = 0.0
                    all_f1_weights = sum(all_weights)
                    if all_f1_weights > 0:
                        for cur_f1, cur_weight in zip(all_f1, all_weights):
                            sum_f1 = sum_f1 + cur_f1*cur_weight
                        overall_micro_f1 = sum_f1 / all_f1_weights
                        print ('Overall weighted f1', overall_micro_f1)
                    else:
                        overall_micro_f1 = 0.0

                    rounded_f1 = round(overall_micro_f1, 2)
                    html_file.write(
                    "<b> Overall Micro F1:</b><br/> %0.2f <br/><br/><br/>" % (rounded_f1))

                    score_file.write("%s_microf1: %0.12f\n" % (overall_prefix, overall_micro_f1))
            
        except Exception as inst:
            score = missing_score
            print(
                "======= Score=ERROR =======")
            html_txt = html_txt = '''
                    <font color='red'>
                        <h3> Error in processing %s </h3>
                    </font><br/> %s
                    ''' % (pred_file, str(inst))
            # html_file.write(
            #     "======= Score=ERROR =======\n")
            print(inst)

    html_file.close()
    score_file.close()
    
    score_file = open(os.path.join(output_dir, 'scores.txt'), 'r')
    html_file = open(os.path.join(output_dir, 'scores.html'), 'r')
    print ('Final Score and HTML dir', output_dir)
    print ('Final score content', score_file.read())
    print ('Final HTML content', html_file.read())
    html_file.close()
    score_file.close()